########################################################################################
###
### SUPPORTING MATERIAL FOR THE ARTICLE
### Baraldo S., Ieva F., Paganoni A. M., Vitelli V., "Outcome prediction for heart 
### failure telemonitoring via generalized linear models with functional covariates", 
### Scandinavian Journal of Statistics (2012)
###
###	Author: Stefano Baraldo
###	Date:   7 June 2012
### 
###	Functions used in GLMHF_Supp.R. Source before running the main script
###
########################################################################################

### UTILITY FUNCTIONS ###

########################################################################################
### basis: returns the evaluations of a b-spline basis on a vector of points x
###
### Inputs:
###     x:  vector of abscissa values for evaluation
### knots:  knots of the basis
###   deg:  polynomial degree of the basis
###
### Outputs:
###         a matrix with the evaluations in values of x (rows) of the B-splines basis
###         functions (columns)
########################################################################################
basis <- function(x, knots, deg=2){
  return(bs(x,knots=knots[2:(length(knots)-1)],degree=deg,Boundary.knots=c(knots[1],knots[length(knots)]),intercept=TRUE))
}

########################################################################################
### int_fun: returns the primitive integral F of a function f on a grid, using the 
###          trapezoidal rule
###
### Inputs:
###     f:  function to be integrated
###     x:  vector of grid points for evaluation of the primitive
###
### Outputs:
###         vector of evaluations of F(t) - F(x[1]), with t in x[1], ..., x[length(x)]
########################################################################################
int_fun <- function(f, x, ...){
  nx <- length(x)
  return(c(0, cumsum(diff(x)*(f(x[-1], ...)+f(x[-nx], ...))/2)))
}

########################################################################################
### increMean: mean of a set of functions, computed by summation of average increments
###
### Inputs:
###     yc: matrix of observed functions (columns) evaluated on a grid (rows)
###
### Outputs:
###    ymi: mean function
########################################################################################
increMean <- function(yc){
  ymi <- mean(yc[1,],na.rm=TRUE)
  ydiff <- apply(X=yc,MARGIN=2,FUN="diff")
  ymi <- cumsum(c(ymi,apply(X=ydiff,MARGIN=1,FUN="mean",na.rm=TRUE)))
  return(ymi)
}